// **********************************************************************
//
// Copyright (c) 2003-2004 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <Ice/Application.h>
#include <IceStorm/IceStorm.h>

#include <Clock.h>

using namespace std;

class Publisher : public Ice::Application
{
public:

	virtual int run(int, char*[]);
};

int
main(int argc, char* argv[])
{
	Publisher app;
	return app.main(argc, argv, "config");
}

int
Publisher::run(int argc, char* argv[])
{
	Ice::PropertiesPtr properties = communicator()->getProperties();

	const string proxyProperty = "IceStorm.TopicManager.Proxy";
	string proxy = properties->getProperty(proxyProperty);
	if(proxy.empty())
	{
		cerr << appName() << ": property `" << proxyProperty << "' not set" << endl;
		return EXIT_FAILURE;
	}

	Ice::ObjectPrx base = communicator()->stringToProxy(proxy);
	IceStorm::TopicManagerPrx manager = IceStorm::TopicManagerPrx::checkedCast(base);
	if(!manager)
	{
		cerr << appName() << ": invalid proxy" << endl;
		return EXIT_FAILURE;
	}

	//
	// Retrieve the topic named "time".
	//
	IceStorm::TopicPrx topic;
	try
	{
		topic = manager->retrieve("time");
	}
	catch(const IceStorm::NoSuchTopic& e)
	{
		cerr << appName() << ": " << e << " name: " << e.name << endl;
		return EXIT_FAILURE;
	}
	assert(topic);

	//
	// Get the topic's publisher object, verify that it supports
	// the Clock type, and create a oneway Clock proxy (for efficiency
	// reasons).
	//
	Ice::ObjectPrx obj = topic->getPublisher();
	if(!obj->ice_isDatagram())
	{
		obj = obj->ice_oneway();
	}
	ClockPrx clock = ClockPrx::uncheckedCast(obj);

	char hostname[256];
	gethostname(hostname,255);

	struct Message msg;

	msg.hostid = hostname;
	for(int i=0; i<512; i++)
		msg.buf.push_back('a');

	for(;;) // enddless loop, for performance testing
	{
		cout << ".";
		Sleep(30); //ms

		msg.timestamp = IceUtil::Time::now().toMicroSeconds();

		try
		{
			for(int i=0; i<1; i++)
			{
				clock->tick(msg);
			}

		}
		catch(const Ice::Exception& ex)
		{
			cerr << ex << endl;
		}
	}


	return EXIT_SUCCESS;
}
